#!/usr/bin/python
# Filename: __init__.py

from dtrange import *

# selection procedures
from scisel import *

# apply a function repeatedly on a multidimensional matrix
from scimulti import *

# statistics calculation
from scistat import *

# various
from sciop import *
